@@ -8,12 +8,13 @@ use icu_provider::prelude::DataError;
 
 /// A list of possible error outcomes for the [`PluralRules`](crate::PluralRules) struct.
 ///
-#[derive(Display, Debug)]
+#[derive(Display, Debug, Clone, Copy)]
+#[non_exhaustive]
 pub enum PluralRulesError {
     /// A parsing error for the plural rules.
     #[displaydoc("Parser error: {0}")]
     Parser(ParserError),
-    /// An error originating inside of the [`DataProvider`](icu_provider::DataProvider)
+    /// An error originating from [`icu_provider`].
     #[displaydoc("Data provider error: {0}")]
     DataProvider(DataError),
 }
