@@ -24,14 +24,12 @@
 //! # Examples
 //!
 //! ```
-//! use icu::locid::macros::langid;
+//! use icu::locid::locale;
 //! use icu::plurals::{PluralRules, PluralRuleType, PluralCategory};
 //!
-//! let lid = langid!("en");
-//!
 //! let provider = icu_testdata::get_provider();
 //!
-//! let pr = PluralRules::try_new(lid, &provider, PluralRuleType::Cardinal)
+//! let pr = PluralRules::try_new(locale!("en"), &provider, PluralRuleType::Cardinal)
 //!     .expect("Failed to construct a PluralRules struct.");
 //!
 //! assert_eq!(pr.select(5_usize), PluralCategory::Other);
@@ -65,7 +63,19 @@
 //! [`Language Plural Rules`]: https://unicode.org/reports/tr35/tr35-numbers.html#Language_Plural_Rules
 //! [`CLDR`]: http://cldr.unicode.org/
 
+// https://github.com/unicode-org/icu4x/blob/main/docs/process/boilerplate.md#library-annotations
 #![cfg_attr(not(any(test, feature = "std")), no_std)]
+#![cfg_attr(
+    not(test),
+    deny(
+        clippy::indexing_slicing,
+        clippy::unwrap_used,
+        clippy::expect_used,
+        clippy::panic,
+        clippy::exhaustive_structs,
+        clippy::exhaustive_enums
+    )
+)]
 
 extern crate alloc;
 
@@ -79,12 +89,15 @@ pub use error::PluralRulesError;
 use icu_locid::Locale;
 use icu_provider::prelude::*;
 pub use operands::PluralOperands;
-use provider::PluralRulesV1Marker;
+use provider::CardinalV1Marker;
+use provider::ErasedPluralRulesV1Marker;
+use provider::OrdinalV1Marker;
 use rules::runtime::test_rule;
 
 /// A type of a plural rule which can be associated with the [`PluralRules`] struct.
 ///
 #[derive(Debug, PartialEq, Eq, Clone, Copy, Hash)]
+#[non_exhaustive]
 pub enum PluralRuleType {
     /// Cardinal plural forms express quantities of units such as time, currency or distance,
     /// used in conjunction with a number expressed in decimal digits (i.e. "2", not "two").
@@ -123,24 +136,22 @@ pub enum PluralRuleType {
 /// # Examples
 ///
 /// ```
-/// use icu::locid::macros::langid;
+/// use icu::locid::locale;
 /// use icu::plurals::{PluralRules, PluralRuleType, PluralCategory};
 /// use icu_provider::inv::InvariantDataProvider;
 ///
-/// let lid = langid!("en");
-///
 /// let dp = InvariantDataProvider;
 ///
-/// let pr = PluralRules::try_new(lid, &dp, PluralRuleType::Cardinal)
+/// let pr = PluralRules::try_new(locale!("en"), &dp, PluralRuleType::Cardinal)
 ///     .expect("Failed to construct a PluralRules struct.");
 ///
 /// assert_eq!(pr.select(5_usize), PluralCategory::Other);
 /// ```
 #[derive(Debug, PartialEq, Eq, Clone, Copy, Hash, Ord, PartialOrd)]
-#[cfg_attr(
-    feature = "provider_serde",
-    derive(serde::Serialize, serde::Deserialize)
-)]
+#[cfg_attr(feature = "datagen", derive(serde::Serialize))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
+#[allow(clippy::exhaustive_enums)] // this type is mostly stable. new categories may potentially be added in the future,
+                                   // but at a cadence slower than the ICU4X release cycle
 pub enum PluralCategory {
     /// CLDR "zero" plural category. Used in Arabic and Latvian, among others.
     ///
@@ -252,15 +263,13 @@ impl PluralCategory {
 /// # Examples
 ///
 /// ```
-/// use icu::locid::macros::langid;
+/// use icu::locid::locale;
 /// use icu::plurals::{PluralRules, PluralRuleType, PluralCategory};
 /// use icu_provider::inv::InvariantDataProvider;
 ///
-/// let lid = langid!("en");
-///
 /// let dp = InvariantDataProvider;
 ///
-/// let pr = PluralRules::try_new(lid, &dp, PluralRuleType::Cardinal)
+/// let pr = PluralRules::try_new(locale!("en"), &dp, PluralRuleType::Cardinal)
 ///     .expect("Failed to construct a PluralRules struct.");
 ///
 /// assert_eq!(pr.select(5_usize), PluralCategory::Other);
@@ -271,7 +280,7 @@ impl PluralCategory {
 /// [`Plural Category`]: PluralCategory
 pub struct PluralRules {
     _locale: Locale,
-    rules: DataPayload<PluralRulesV1Marker>,
+    rules: DataPayload<ErasedPluralRulesV1Marker>,
 }
 
 impl PluralRules {
@@ -282,11 +291,11 @@ impl PluralRules {
     /// # Examples
     ///
     /// ```
-    /// use icu::locid::macros::langid;
+    /// use icu::locid::locale;
     /// use icu::plurals::{PluralRules, PluralRuleType};
     /// use icu_provider::inv::InvariantDataProvider;
     ///
-    /// let lid = langid!("en");
+    /// let lid = locale!("en");
     ///
     /// let dp = InvariantDataProvider;
     ///
@@ -294,31 +303,109 @@ impl PluralRules {
     /// ```
     ///
     /// [`type`]: PluralRuleType
-    /// [`data provider`]: icu_provider::DataProvider
+    /// [`data provider`]: icu_provider
     pub fn try_new<T: Into<Locale>, D>(
         locale: T,
         data_provider: &D,
         rule_type: PluralRuleType,
     ) -> Result<Self, PluralRulesError>
     where
-        D: DataProvider<PluralRulesV1Marker> + ?Sized,
+        D: ResourceProvider<CardinalV1Marker> + ResourceProvider<OrdinalV1Marker> + ?Sized,
+    {
+        match rule_type {
+            PluralRuleType::Cardinal => Self::try_new_cardinal(locale, data_provider),
+            PluralRuleType::Ordinal => Self::try_new_ordinal(locale, data_provider),
+        }
+    }
+
+    /// Constructs a new `PluralRules` for a given locale for cardinal numbers.
+    ///
+    /// Cardinal plural forms express quantities of units such as time, currency or distance,
+    /// used in conjunction with a number expressed in decimal digits (i.e. "2", not "two").
+    ///
+    /// For example, English has two forms for cardinals:
+    ///
+    /// * [`One`]: `1 day`
+    /// * [`Other`]: `0 days`, `2 days`, `10 days`, `0.3 days`
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::locale;
+    /// use icu::plurals::{PluralRules, PluralCategory};
+    ///
+    /// let dp = icu_testdata::get_provider();
+    ///
+    /// let rules = PluralRules::try_new_cardinal(locale!("ru"), &dp)
+    ///     .expect("Data should be present");
+    ///
+    /// assert_eq!(rules.select(2_usize), PluralCategory::Few);
+    /// ```
+    ///
+    /// [`One`]: PluralCategory::One
+    /// [`Other`]: PluralCategory::Other
+    pub fn try_new_cardinal<T: Into<Locale>, D>(
+        locale: T,
+        data_provider: &D,
+    ) -> Result<Self, PluralRulesError>
+    where
+        D: ResourceProvider<CardinalV1Marker> + ?Sized,
     {
         let locale = locale.into();
-        let key = match rule_type {
-            PluralRuleType::Cardinal => provider::key::CARDINAL_V1,
-            PluralRuleType::Ordinal => provider::key::ORDINAL_V1,
-        };
         let rules = data_provider
-            .load_payload(&DataRequest {
-                resource_path: ResourcePath {
-                    key,
-                    options: ResourceOptions {
-                        variant: None,
-                        langid: Some(locale.clone().into()),
-                    },
-                },
+            .load_resource(&DataRequest {
+                options: locale.clone().into(),
+                metadata: Default::default(),
             })?
-            .take_payload()?;
+            .take_payload()?
+            .cast();
+        Self::new(locale, rules)
+    }
+
+    /// Constructs a new `PluralRules` for a given locale for ordinal numbers.
+    ///
+    /// Ordinal plural forms denote the order of items in a set and are always integers.
+    ///
+    /// For example, English has four forms for ordinals:
+    ///
+    /// * [`One`]: `1st floor`, `21st floor`, `101st floor`
+    /// * [`Two`]: `2nd floor`, `22nd floor`, `102nd floor`
+    /// * [`Few`]: `3rd floor`, `23rd floor`, `103rd floor`
+    /// * [`Other`]: `4th floor`, `11th floor`, `96th floor`
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::locale;
+    /// use icu::plurals::{PluralRules, PluralCategory};
+    ///
+    /// let dp = icu_testdata::get_provider();
+    ///
+    /// let rules = PluralRules::try_new_ordinal(locale!("ru"), &dp)
+    ///     .expect("Data should be present");
+    ///
+    /// assert_eq!(rules.select(2_usize), PluralCategory::Other);
+    /// ```
+    ///
+    /// [`One`]: PluralCategory::One
+    /// [`Two`]: PluralCategory::Two
+    /// [`Few`]: PluralCategory::Few
+    /// [`Other`]: PluralCategory::Other
+    pub fn try_new_ordinal<T: Into<Locale>, D>(
+        locale: T,
+        data_provider: &D,
+    ) -> Result<Self, PluralRulesError>
+    where
+        D: ResourceProvider<OrdinalV1Marker> + ?Sized,
+    {
+        let locale = locale.into();
+        let rules = data_provider
+            .load_resource(&DataRequest {
+                options: locale.clone().into(),
+                metadata: Default::default(),
+            })?
+            .take_payload()?
+            .cast();
         Self::new(locale, rules)
     }
 
@@ -327,15 +414,13 @@ impl PluralRules {
     /// # Examples
     ///
     /// ```
-    /// use icu::locid::macros::langid;
+    /// use icu::locid::locale;
     /// use icu::plurals::{PluralRules, PluralRuleType, PluralCategory};
     /// use icu_provider::inv::InvariantDataProvider;
     ///
-    /// let lid = langid!("en");
-    ///
     /// let dp = InvariantDataProvider;
     ///
-    /// let pr = PluralRules::try_new(lid, &dp, PluralRuleType::Cardinal)
+    /// let pr = PluralRules::try_new(locale!("en"), &dp, PluralRuleType::Cardinal)
     ///     .expect("Failed to construct a PluralRules struct.");
     ///
     /// match pr.select(1_usize) {
@@ -357,16 +442,14 @@ impl PluralRules {
     ///
     /// ```
     /// use std::convert::TryFrom;
-    /// use icu::locid::macros::langid;
+    /// use icu::locid::locale;
     /// use icu::plurals::{PluralRules, PluralRuleType};
     /// use icu::plurals::{PluralCategory, PluralOperands};
     /// use icu_provider::inv::InvariantDataProvider;
     /// #
-    /// # let lid = langid!("en");
-    /// #
     /// # let dp = InvariantDataProvider;
     /// #
-    /// # let pr = PluralRules::try_new(lid, &dp, PluralRuleType::Cardinal)
+    /// # let pr = PluralRules::try_new(locale!("en"), &dp, PluralRuleType::Cardinal)
     /// #     .expect("Failed to construct a PluralRules struct.");
     ///
     /// let operands = PluralOperands::try_from(-5)
@@ -411,15 +494,13 @@ impl PluralRules {
     /// # Examples
     ///
     /// ```
-    /// use icu::locid::macros::langid;
+    /// use icu::locid::locale;
     /// use icu::plurals::{PluralRules, PluralRuleType, PluralCategory};
     /// use icu_provider::inv::InvariantDataProvider;
     ///
-    /// let lid = langid!("fr");
-    ///
     /// let dp = icu_testdata::get_provider();
     ///
-    /// let pr = PluralRules::try_new(lid, &dp, PluralRuleType::Cardinal)
+    /// let pr = PluralRules::try_new(locale!("fr"), &dp, PluralRuleType::Cardinal)
     ///     .expect("Failed to construct a PluralRules struct.");
     ///
     /// let mut categories = pr.categories();
@@ -453,9 +534,9 @@ impl PluralRules {
 
     /// Lower-level constructor that allows constructing a [`PluralRules`] directly from
     /// data obtained from a provider.
-    pub fn new<T: Into<Locale>>(
+    fn new<T: Into<Locale>>(
         locale: T,
-        rules: DataPayload<PluralRulesV1Marker>,
+        rules: DataPayload<ErasedPluralRulesV1Marker>,
     ) -> Result<Self, PluralRulesError> {
         let locale = locale.into();
         Ok(Self {
