@@ -71,7 +71,8 @@ use fixed_decimal::FixedDecimal;
 ///    c: 0,
 /// }), FixedDecimal::from(12345).multiplied_pow10(-2).map(|d| (&d).into()))
 /// ```
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy, PartialEq, Default)]
+#[allow(clippy::exhaustive_structs)] // mostly stable, new operands may be added at the cadence of ICU's release cycle
 pub struct PluralOperands {
     /// Integer value of input
     pub i: u64,
@@ -101,6 +102,7 @@ impl PluralOperands {
 }
 
 #[derive(Display, Debug, PartialEq, Eq)]
+#[non_exhaustive]
 pub enum OperandsError {
     /// Input to the Operands parsing was empty.
     #[displaydoc("Input to the Operands parsing was empty")]
@@ -128,7 +130,9 @@ impl From<std::io::Error> for OperandsError {
 
 fn get_exponent(input: &str) -> Result<(&str, usize), OperandsError> {
     if let Some(e_idx) = input.find('e') {
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
         let e = usize::from_str(&input[e_idx + 1..])?;
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
         Ok((&input[..e_idx], e))
     } else {
         Ok((input, 0))
@@ -153,7 +157,9 @@ impl FromStr for PluralOperands {
             fraction_digits,
             exponent,
         ) = if let Some(sep_idx) = abs_str.find('.') {
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
             let int_str = &abs_str[..sep_idx];
+            #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
             let (dec_str, exponent) = get_exponent(&abs_str[(sep_idx + 1)..])?;
 
             let integer_digits = u64::from_str(int_str)?;
